/**
 * TemperatureModule is the web service class
 * Copyright 2006, The MathWorks, Inc. 
 */ 

package com.mathworks.toolbox.javabuilder.examples;

import java.io.*;
import java.util.*;

import com.mathworks.toolbox.javabuilder.*;
import temperatureComp.*;

public class TemperatureModule {

 /**
 * Converts temperature(s) from one format to another. This method calls the 
 * underlying Java Builder component's convertTemperature method.
 *
 * @param tempIn The input temperature(s) to convert. This can be of type double, 
 *			array of doubles, TemperatureCellArray, or TemperatureStruct
 * @param tempType The type to convert to: must either be 'f' for Fahrenheit or
 *			'c' for Celsius. The input temperatures are assumed to be in the 
 *			complimentary format.
 * @return Object that contains the converted temperatures in the same format
 *			specified in tempIn
 */
  public Object convertTemp (Object tempIn, String tempType) throws Exception {
       
    Object[] result = null;
	temperatureClass aClass = null;

	try
	{
		/* create instance of component */
    	aClass = new temperatureClass("D:\\Applications\\Apache Software Foundation\\Tomcat 5.5\\webapps\\axis\\WEB-INF\\lib");

	    /* tempIn can be 1 of 4 types; use reflection to find out which type is being passed in */

	    /* scalar input passed as double */
	    if(tempIn instanceof Double)
	    {
	    	/* pass input directly into component method as is */
	    	result = aClass.convertTemperature(1, tempIn, tempType);
	    	
	   		/* return underlying Java representation of result */
		    return ((MWArray)result[0]).toArray();
	    }
	    
	    /* array of temperatures */
	    else if(tempIn instanceof double[][])
	    {
	    	/* pass input as MWNumericArray */
	    	MWNumericArray tempInMW = new MWNumericArray(tempIn, MWClassID.DOUBLE);
	    	result = aClass.convertTemperature(1, tempInMW, tempType);
	    	MWArray.disposeArray(tempInMW);
	    	
	   		/* return underlying Java representation of result */
		    return ((MWArray)result[0]).toArray();
	    }
	    
	    /* cell array of temperatures (custom-built class TemperatureCellArray)
	     * can be jagged or not 
	     */
	    else if(tempIn instanceof TemperatureCellArray)
	    {
	    	/* get array of temperatures from object */
	    	double[][] temperatures = ((TemperatureCellArray)tempIn).getTemperatures();

			/* create main MWCellArray to store this information */
	    	MWCellArray tempInCell = new MWCellArray(1,temperatures.length);
	    	
	    	/* create separate MWCellArray elements for each array element of temperatures */
	    	for(int i=0; i<temperatures.length; i++)
	    	{	   		
	    		MWCellArray tempInCellAtI = new MWCellArray(1,temperatures[i].length);
	    		
	    		/* fill the sub-cellarray with each temperature as a separate MWNumericArray type*/
	    		/* this is just ONE way to pass the data in; this method preserves the jagged structure of the input array */
	    		/* You could modify the code to pass just one MWCellArray containing 3 MWNumericArrays instead */
	    		for(int j=0; j<temperatures[i].length; j++)
	    		{	
	    			MWNumericArray arrayAtJ = new MWNumericArray(temperatures[i][j], MWClassID.DOUBLE);
	    			tempInCellAtI.set(j+1, arrayAtJ);
	    			MWArray.disposeArray(arrayAtJ);
	    		}
	    		tempInCell.set(i+1, tempInCellAtI);
	    		MWArray.disposeArray(tempInCellAtI);
	    	}
	    	
	    	/* pass in MWCellArray to temperature conversion method */
	    	result = aClass.convertTemperature(1, tempInCell, tempType);
	    	
	    	/* Now we will reconstruct a TemperatureCellArray to pass back to the client */
	    	/* This code uses some basic MWCellArray methods */
	    		    	
	    	/* get the result in MWCellArray format */
   			MWCellArray resultCell = (MWCellArray)result[0];
   			
   			/* gather information about the dimensions (this should match with the input) */
   			int dims[] = resultCell.getDimensions();
   			
   			/* create a TemperatureCellArray to return */
   			TemperatureCellArray returnTemperatures = new TemperatureCellArray(dims[1], dims);
   			
   			/* get each sub-cell and assign it to the TemperatureCellArray output */
   			for(int k=0; k<dims[1]; k++)
   			{
   				int indices[] = {1,k+1};
   				MWArray resultCellAtK = (resultCell.getCell(k+1));
   				
   				/* get underlying data in Java datatype */
   				double[][] arrayAtK = (double[][])(resultCellAtK.toArray());
   				returnTemperatures.setIndex(k, arrayAtK[0]);  				
   			}   			
   			
   			/* Dispose of temporary variables */
 	    	MWArray.disposeArray(tempInCell);	
 	     	    	    	
   			return returnTemperatures; 
   		}   		
   		
   		/* struct array of temperatures (custom-built class TemperatureStruct)
	     * can contain jagged temperatures */		
	    else if(tempIn instanceof TemperatureStruct)
	    {
	    	/* get temperature(s) from object */
	    	Object temperatures = ((TemperatureStruct)tempIn).getTemperatures();

			/* create main MWStructArray to store this information */
			String[] fieldnames = {"temperatures"};
	    	MWStructArray tempInStruct = new MWStructArray(1, 1, fieldnames);
	    	
	    	if(temperatures instanceof Double)
		    {
		    	/* assign Double directly */
		    	tempInStruct.set("temperatures", 1, temperatures);
		    	result = aClass.convertTemperature(1, tempInStruct, tempType);
		    	
   		    	/* get the result in MWStructArray format */
	   			MWStructArray resultStruct = (MWStructArray)result[0];

				/* get the field containing the converted temperatures */
				MWArray resultStructAsMWArray = resultStruct.getField("temperatures", 1);
				
	   			/* create a TemperatureStruct to return */
	   			TemperatureStruct returnTemperatures = new TemperatureStruct(resultStructAsMWArray.toArray());
	   			
	   			return returnTemperatures;			   
		    }
		    
		    /* array of temperatures */
		    else if(temperatures instanceof double[][])
		    {
		    	/* For variety, instead of constructing MWNumericArrays here, 
		    	 * let's pass input as double[][]. Note that if the 
		    	 * input is jagged and is passed into the component method call 
		    	 * as is, it will be converted to a square matrix inside MATLAB, 
		    	 * and 0's will be inserted in all the "holes" */
		    	tempInStruct.set("temperatures", 1, temperatures);
		    	result = aClass.convertTemperature(1, tempInStruct, tempType);
		    	
   		    	/* get the result in MWStructArray format */
	   			MWStructArray resultStruct = (MWStructArray)result[0];

				/* get the field containing the converted temperatures */
				MWArray resultStructAsMWArray = resultStruct.getField("temperatures", 1);
				
	   			/* create a TemperatureStruct to return */
	   			TemperatureStruct returnTemperatures = new TemperatureStruct(resultStructAsMWArray.toArray());
	   			
	   			return returnTemperatures;			   
		    }	    
	    	    	
	    	/* cell array of temperatures (custom-built class TemperatureCellArray)
	    	 * can be jagged or not
	    	 * NOTE: This code block is unmodified from the previous check for 
	    	 * "TemperatureCellArray" except that the constructed MWCellArray is 
	    	 * additionally placed inside an MWStructArray
	    	 */
		    else if(temperatures instanceof TemperatureCellArray){
		    	
		    	/* get array of temperatures from object */
		    	double[][] temperaturesInCell = ((TemperatureCellArray)temperatures).getTemperatures();
	
				/* create main MWCellArray to store this information */
		    	MWCellArray tempInCell = new MWCellArray(1,temperaturesInCell.length);	    	
	    	
		    	/* create separate MWCellArray elements for each array element of temperatures */
		    	for(int i=0; i<temperaturesInCell.length; i++)
		    	{	   		
		    		MWCellArray tempInCellAtI = new MWCellArray(1,temperaturesInCell[i].length);
		    		
		    		/* fill the sub-cellarray with each temperature as a separate MWNumericArray type*/
		    		/* this is just ONE way to pass the data in; this method preserves the jagged structure of the input array */
		    		/* You could modify the code to pass just one MWCellArray containing 3 MWNumericArrays instead */
		    		for(int j=0; j<temperaturesInCell[i].length; j++)
		    		{	
		    			MWNumericArray arrayAtJ = new MWNumericArray(temperaturesInCell[i][j], MWClassID.DOUBLE);
		    			tempInCellAtI.set(j+1, arrayAtJ);
		    			MWArray.disposeArray(arrayAtJ);
		    		}
		    		tempInCell.set(i+1, tempInCellAtI);
		    		MWArray.disposeArray(tempInCellAtI);
		    	}
		    	
		    	/* assign MWCellArray as struct input */
		    	tempInStruct.set("temperatures", 1, tempInCell);
		    	
		    	/* pass in MWStructArray to temperature conversion method */
		    	result = aClass.convertTemperature(1, tempInStruct, tempType);
		    	
		    	/* Now we will reconstruct a TemperatureStruct to pass back to the client */
		    	
		    	/* get the result in MWStructArray format */
	   			MWStructArray resultStruct = (MWStructArray)result[0];
	   			
		    	/* get the underlying array in MWCellArray format */
	   			MWCellArray resultCell = (MWCellArray)(resultStruct.getField("temperatures", 1));

	   			/* gather information about the dimensions (this should match with the input) */
	   			int dims[] = resultCell.getDimensions();
	   			
	   			/* create a TemperatureStruct to return */
				TemperatureStruct returnTemperatures = new TemperatureStruct();   			

	   			/* create a TemperatureCellArray to embed in the struct to return */
	   			TemperatureCellArray returnTempsCell = new TemperatureCellArray(dims[1], dims);
	   			
   				/* get each sub-cell and assign it to the TemperatureCellArray output */
	   			for(int k=0; k<dims[1]; k++)
	   			{
	   				int indices[] = {1,k+1};
	   				MWArray resultCellAtK = (resultCell.getCell(k+1));
	   				
	   				/* get underlying data in Java datatype */
	   				double[][] arrayAtK = (double[][])(resultCellAtK.toArray());
	   				returnTempsCell.setIndex(k, arrayAtK[0]);  				
	   			}	   			
	   			
   				/* Dispose of temporary variables */
	 	    	MWArray.disposeArray(tempInCell);	
	 	    	
	 	    	/* assign the cellarray to the structure */
	 	    	returnTemperatures.setTemperatures(returnTempsCell);
    	
	   			return returnTemperatures; 
	   		}
   		}
   	}
	
	catch(Exception e)
	{
		System.out.println(e.getStackTrace());
		throw e;
	}
		
	finally
	{
		/* Free native resources */
	    if (aClass != null)
	       aClass.dispose();
	}

	return null;
  }

}
